/* greeterwindow.h
 * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301, USA.
**/
#ifndef GREETERWINDOW_H
#define GREETERWINDOW_H

#include <QWidget>
#include <QStackedLayout>
#include <QSharedPointer>
#include <QtConcurrent/QtConcurrentRun>
#include <QTranslator>
#include <QLocale>
#include <QScrollArea>
#include <QTimer>
#include "loginwindow.h"
#include "sessionwindow.h"
#include "usersmodel.h"
#include "greeterwrapper.h"
#include "ktooltip.h"
#include "surewidget.h"
#include "xeventmonitor.h"
#include "batterywidget.h"

class UsersView;
class PowerManager;
class QProcess;
class QGraphicsOpacityEffect;
class Configuration;
class VirtualKeyboard;
class LanguageWidget;
class Language;
class KylinNM;

enum TABAT {
    EMPTY = -1,
    LINEEDIT = 0,
    ENTERBTN,
    BIOBTN,
    BOTTMBTN,
    POWERMANAGER,
};

enum HORIZONBTN {
    BATTERYBTN = 0,
    SWITCHBTN,
    NETWORKBTN,
    KEYBOARDBTN,
    POWERBTN,
    SESSIONBTN,
};

class GreeterWindow : public QWidget
{
    Q_OBJECT
public:
    GreeterWindow(QWidget *parent = 0);
    void initUI();
    QString guessBackground();
    void setRootWindow(); 
    QString getCurUser();
    QString getBatteryIconName();

signals:
    void currentUserChange(QString strUserName);
    void authenticationSucess(QString userName);

protected:
    void resizeEvent(QResizeEvent *event);
    void keyReleaseEvent(QKeyEvent *event);
    bool eventFilter(QObject *obj, QEvent *event);

private slots:
    void onAuthenticationComplete1();
    void onUserSelected(const QModelIndex &);
    void onCurrentUserChanged(const QModelIndex&);
    void timedAutologin();
    void showPowerWnd(bool keynavigation = false);
    void showNetManager(bool keynavigation = false);
    void onClickPassword();
    void showBattery();
    void toolTipShow(const QString &text);
    void toolTipHide();
    void showVirtualKeyboard();
    void showUserMenu();
    void onLanguageChanged(QString languageCode);
    void showSessionWnd();
    void onSessionChanged(const QString &session);
    void onUserChangedByManual(const QString &userName);
    void setWindowVisible();
    void switchToSureDialog(int type);
    void hideSureDialog();
    void confirmClicked();
    void onGlobalKeyPress(const QString &key);
    void onGlobalkeyRelease(const QString &key);
    void resetNavigation();
    void setBatteryIcon(const QString &str);
    void dealMessage(QDBusMessage);
    void hideBottomPlugins();

private:
    void switchWnd(int);
    void setBackground(const QModelIndex &index);
    void setVirkeyboardPos();
    void setWindowPos(QWidget *widget, Qt::Alignment align = Qt::AlignCenter);
    bool sessionIsValid(const QString &session);
    void updateLanguage(QString userName);
    void updateSession(QString userName);
    void setUserWindowVisible(bool visible);
    void refreshTranslate();
    void drawBackground();
    void updateNetIcon(int status);
    QString getAccountBackground(uid_t uid);
    int getNetStatus();
    void key_OB_release(int key);
    void key_tab_release(int key);
    void key_shiftTab_release();
    void key_LR_release(int key);
    void key_enter_release(int key);
    void key_exit_release();
    void setBottomBtnSheet();
    void setCheckedSheet(int type, bool show);
    //tooltip
    Tooltip         *kTooltip = nullptr;
    bool            is_Show = false;
    int height1 ;
    int width1 ;

    UsersView               *m_userWnd;
    LoginWindow             *m_loginWnd;
    SessionWindow           *m_sessionWnd;
    PowerManager             *m_powerWnd;
    QPushButton             *m_keyboardLB;
    QPushButton             *m_powerLB;
    QPushButton		    *m_nmLB = nullptr;
    QPushButton             *m_btnSwitchUser = nullptr;
    QPushButton             *m_sessionLB;
    QPushButton             *m_batteryiconLB = nullptr;
    VirtualKeyboard         *m_virtualKeyboard;
    QScrollArea             *scrollArea = nullptr;
    QWidget                 *scrollContents = nullptr;

    GreeterWrapper          *m_greeter;
    UsersModel              *m_usersModel;
    QAbstractItemModel      *m_sessionsModel;
    Configuration           *m_configuration;
    int                     timeType;
    QString                 dataType;
    bool                     m_sessionHasChanged;

    QTimer 		    *timer;
    QWidget	    	    *widgetTime;
    QLabel		    *lblDate;
    QLabel		    *lblTime;
    QLocale                 local;
    QString                 currentUser = "";
    QFuture<void>           future;
    QFuture<void>           futureNM;
    KylinNM                 *m_kylinNM = nullptr;
    QWidget                 *m_NetManagerWidget;
    BatteryWidget           *mBatteryWidget = nullptr;

    bool		    isNetFinished = false;

    Surewidget              *sureWidget = nullptr;
    int                   powermanagerType;

    bool               is_switchBtn = true;
    bool               is_batteryBtn = true;
    bool               m_isSessionBtnShow = false;
    int                tabAt = LINEEDIT;
    int                horAT = SWITCHBTN;
    int                at_power = false;
    bool               at_plugins = false;

    QDBusInterface *batInterface = nullptr;
    QDBusInterface *iface = nullptr;
    QDBusInterface *dface = nullptr;
};

#endif // GREETERWINDOW_H
